using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace MetaLabLogoProject
{
    public partial class Form1 : Form
    {
        IParticleSystem particleSystem = new MetaLabLogoProject.GravityParticleSystem.GravityParticleSystem();
        Timer t;
        DateTime lastStep;

        public Form1()
        {
            InitializeComponent();
            particleSystemSurface1.BackColor = Color.FromArgb(100, 0, 20, 50);
            particleSystemSurface1.ZoomFactor = 0.5f;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //set parameter texts
            tbZoom.Text = particleSystemSurface1.ZoomFactor.ToString();
            tbColor.Text = particleSystemSurface1.BackColor.A + " " +
                            particleSystemSurface1.BackColor.R + " " +
                            particleSystemSurface1.BackColor.G + " "+ 
                            particleSystemSurface1.BackColor.B;

            //first render
            particleSystemSurface1.ParticleSystem = particleSystem;
            RenderParticleSystem();
            
            t = new Timer();
            t.Interval = 1;
            t.Tick += new EventHandler(stepParticleSystem);
            lastStep = DateTime.Now;
            t.Enabled = true;
        }

        void stepParticleSystem(object sender, EventArgs e)
        {
            DateTime now = DateTime.Now;
            TimeSpan ts = now.Subtract(lastStep);
            lastStep = now;

            particleSystem.Step(ts.Milliseconds / 10.0);
            RenderParticleSystem();
        }

        void RenderParticleSystem()
        {
            particleSystemSurface1.UpdateBuffer();
            particleSystemSurface1.Invalidate();
        }

        private void btnRestart_Click(object sender, EventArgs e)
        {
            particleSystem = new MetaLabLogoProject.GravityParticleSystem.GravityParticleSystem();
            particleSystemSurface1.ParticleSystem = particleSystem;
            particleSystemSurface1.ClearBuffer();
            RenderParticleSystem();
            lastStep = DateTime.Now;
        }

        private void HandleColorEnter(object sender, KeyEventArgs e)
        {
            if (e.KeyCode != Keys.Enter) return;

            string[] split;
            byte[] colorComponents = new byte[4];

            split = tbColor.Text.Split(' ');

            if (split.Length != 4) return;

            for (int i = 0; i < 4; ++i)
            {
		    try {
		colorComponents[i] = byte.Parse(split[i]);
		    }
		    catch {
			    return;
		    }
            }

            particleSystemSurface1.BackColor = Color.FromArgb(colorComponents[0], colorComponents[1], colorComponents[2], colorComponents[3]);
        }

        private void HandleZoomEnter(object sender, KeyEventArgs e)
        {
            if (e.KeyCode != Keys.Enter) return;

            float parsed;

	    try {
            parsed = float.Parse(tbZoom.Text);
	    } catch { return; }
                particleSystemSurface1.ZoomFactor = parsed;
        }
    }
}
