using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace MetaLabLogoProject
{
    public class ParticleSystemSurface : Control
    {
        Bitmap buffer;
        Graphics graphics;
        IParticleSystem particleSystem;
        SolidBrush backgroundBrush;
        RectangleF fillRectangle;
        float zoomFactor = 1, inverseZoomFactor = 1;

        public ParticleSystemSurface()
        {
		// Please uncomment / comment the double buffer flag applicable to your compilter / runtime
		// I'll introduce something better when I have more time at hand
		
            SetStyle(   ControlStyles.AllPaintingInWmPaint | 
                        ControlStyles.UserPaint | 
                        ControlStyles.Opaque | 
        //MS .NET 2.0                ControlStyles.OptimizedDoubleBuffer | //MS .NET 2.0
			ControlStyles.DoubleBuffer |  // MONO
                        ControlStyles.SupportsTransparentBackColor, 
                        true);

            backgroundBrush = new SolidBrush(Color.Black);

            Resize += new EventHandler(HandleResize);
            ParentChanged += new EventHandler(HandleResize);
            BackColorChanged += new EventHandler(HandleBackColorChanged);
        }

        void HandleBackColorChanged(object sender, EventArgs e)
        {
            backgroundBrush.Color = BackColor;
        }

        void HandleResize(object sender, EventArgs e)
        {
            InitializeBuffer();
        }

        /// <summary>
        /// (Re-)Initialize the buffer and graphics objects.
        /// </summary>
        void InitializeBuffer()
        {
            if (graphics != null) graphics.Dispose();
            if (buffer != null) buffer.Dispose();

            if (Width == 0 || Height == 0) return;

            buffer = new Bitmap(Width, Height);
            
            graphics = Graphics.FromImage(buffer);
            graphics.SmoothingMode = SmoothingMode.HighQuality;
            graphics.Clear(Color.Black);
            graphics.TranslateTransform(Width / 2, Height / 2);
            graphics.ScaleTransform(zoomFactor, zoomFactor);
            fillRectangle = new RectangleF(-Width / 2 * inverseZoomFactor, -Height / 2 * inverseZoomFactor, Width*inverseZoomFactor, Height*inverseZoomFactor);
        }

        public void ClearBuffer()
        {
            graphics.Clear(Color.Black);
        }

        public IParticleSystem ParticleSystem { get { return particleSystem; } set { particleSystem = value; } }
        public float ZoomFactor
        {
            get { return zoomFactor; }
            set
            {
                zoomFactor = value;
                inverseZoomFactor = 1 / value;
                if (graphics != null) InitializeBuffer();
            }
        }

        /// <summary>
        /// Updatesthe display buffer without redrawing the ParticleSystem
        /// on screen.
        /// </summary>
        /// <remarks>
        /// I hope this can be user for file output and to give various
        /// effects to be updated more frequently than the screen itself.
        /// 
        /// This all depends on how much of an overhead rendering to the screen
        /// opposed to rendering to a bitmap really has.
        /// </remarks>
        public void UpdateBuffer() {
            graphics.FillRectangle(backgroundBrush, fillRectangle);
            particleSystem.Render(graphics);
        }

        /// <summary>
        /// Flushes pending render operations on the buffer and
        /// draws the buffer on screen.
        /// </summary>
        /// <remarks>
        /// To obtain meaningful output it is assumed that UpdateBuffer and
        /// OnPaint are not used concurrently. This could be done by restricting
        /// use of these methods to the UI thread.
        /// </remarks>
        protected override void OnPaint(PaintEventArgs pe)
        {
            graphics.Flush();
            pe.Graphics.DrawImageUnscaled(buffer, 0,0);
        }
    }
}
